@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2014-2024 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -10,8 +10,8 @@
  * copies of the Software, and to permit persons to whom the Software is
  * furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -37,7 +37,6 @@
 #include <QRegularExpressionMatch>
 #include <QRegularExpressionMatchIterator>
 #include <QScrollBar>
-#include <QSettings>
 #include <QTextBlock>
 #include <QTimer>
 #include <QWheelEvent>
@@ -95,22 +94,23 @@ QMarkdownTextEdit::QMarkdownTextEdit(QWidget *parent, bool initHighlighter)
             &QMarkdownTextEdit::adjustRightMargin);
     connect(this, &QPlainTextEdit::cursorPositionChanged, this,
             &QMarkdownTextEdit::centerTheCursor);
-    connect(verticalScrollBar(), &QScrollBar::valueChanged, this, [this](int) {
-        _lineNumArea->update();
-    });
+    connect(verticalScrollBar(), &QScrollBar::valueChanged, this,
+            [this](int) { _lineNumArea->update(); });
     connect(this, &QPlainTextEdit::cursorPositionChanged, this, [this]() {
         _lineNumArea->update();
 
-        auto oldArea = blockBoundingGeometry(_textCursor.block()).translated(contentOffset());
+        auto oldArea = blockBoundingGeometry(_textCursor.block())
+                           .translated(contentOffset());
         _textCursor = textCursor();
-        auto newArea = blockBoundingGeometry(_textCursor.block()).translated(contentOffset());
+        auto newArea = blockBoundingGeometry(_textCursor.block())
+                           .translated(contentOffset());
         auto areaToUpdate = oldArea | newArea;
         viewport()->update(areaToUpdate.toRect());
     });
-    connect(document(), &QTextDocument::blockCountChanged,
-            this, &QMarkdownTextEdit::updateLineNumberAreaWidth);
-    connect(this, &QPlainTextEdit::updateRequest,
-            this, &QMarkdownTextEdit::updateLineNumberArea);
+    connect(document(), &QTextDocument::blockCountChanged, this,
+            &QMarkdownTextEdit::updateLineNumberAreaWidth);
+    connect(this, &QPlainTextEdit::updateRequest, this,
+            &QMarkdownTextEdit::updateLineNumberArea);
 
     updateSettings();
 
@@ -126,29 +126,22 @@ void QMarkdownTextEdit::setLineNumbersOtherLineColor(QColor color) {
     _lineNumArea->setOtherLineColor(std::move(color));
 }
 
-void QMarkdownTextEdit::setSearchWidgetDebounceDelay(uint debounceDelay)
-{
+void QMarkdownTextEdit::setSearchWidgetDebounceDelay(uint debounceDelay) {
     _debounceDelay = debounceDelay;
     searchWidget()->setDebounceDelay(_debounceDelay);
 }
 
-void QMarkdownTextEdit::setHighlightCurrentLine(bool set)
-{
+void QMarkdownTextEdit::setHighlightCurrentLine(bool set) {
     _highlightCurrentLine = set;
 }
 
-bool QMarkdownTextEdit::highlightCurrentLine()
-{
-    return _highlightCurrentLine;
-}
+bool QMarkdownTextEdit::highlightCurrentLine() { return _highlightCurrentLine; }
 
-void QMarkdownTextEdit::setCurrentLineHighlightColor(const QColor &color)
-{
+void QMarkdownTextEdit::setCurrentLineHighlightColor(const QColor &color) {
     _currentLineHighlightColor = color;
 }
 
-QColor QMarkdownTextEdit::currentLineHighlightColor()
-{
+QColor QMarkdownTextEdit::currentLineHighlightColor() {
     return _currentLineHighlightColor;
 }
 
@@ -290,12 +283,14 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
             return bracketClosingCheck(QLatin1Char('['), QLatin1Char(']'));
         } else if (keyEvent->key() == Qt::Key_Greater) {
             return bracketClosingCheck(QLatin1Char('<'), QLatin1Char('>'));
-        } else if ((keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) &&
+        } else if ((keyEvent->key() == Qt::Key_Return ||
+                    keyEvent->key() == Qt::Key_Enter) &&
                    keyEvent->modifiers().testFlag(Qt::ShiftModifier)) {
             QTextCursor cursor = this->textCursor();
             cursor.insertText("  \n");
             return true;
-        } else if ((keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) &&
+        } else if ((keyEvent->key() == Qt::Key_Return ||
+                    keyEvent->key() == Qt::Key_Enter) &&
                    keyEvent->modifiers().testFlag(Qt::ControlModifier)) {
             QTextCursor cursor = this->textCursor();
             cursor.movePosition(QTextCursor::EndOfBlock);
@@ -390,7 +385,8 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
                 }
             }
             return QPlainTextEdit::eventFilter(obj, event);
-        } else if (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) {
+        } else if (keyEvent->key() == Qt::Key_Return ||
+                   keyEvent->key() == Qt::Key_Enter) {
             return handleReturnEntered();
         } else if ((keyEvent->key() == Qt::Key_F3)) {
             _searchWidget->doSearch(
@@ -412,26 +408,30 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
             moveTextUpDown(true);
             return true;
 #ifdef Q_OS_MAC
-        // https://github.com/pbek/QOwnNotes/issues/1593
-        // https://github.com/pbek/QOwnNotes/issues/2643
+            // https://github.com/pbek/QOwnNotes/issues/1593
+            // https://github.com/pbek/QOwnNotes/issues/2643
         } else if (keyEvent->key() == Qt::Key_Home) {
             QTextCursor cursor = textCursor();
             // Meta is Control on macOS
             cursor.movePosition(
-                keyEvent->modifiers().testFlag(Qt::MetaModifier) ?
-                    QTextCursor::Start : QTextCursor::StartOfLine,
-                keyEvent->modifiers().testFlag(Qt::ShiftModifier) ?
-                    QTextCursor::KeepAnchor : QTextCursor::MoveAnchor);
+                keyEvent->modifiers().testFlag(Qt::MetaModifier)
+                    ? QTextCursor::Start
+                    : QTextCursor::StartOfLine,
+                keyEvent->modifiers().testFlag(Qt::ShiftModifier)
+                    ? QTextCursor::KeepAnchor
+                    : QTextCursor::MoveAnchor);
             this->setTextCursor(cursor);
             return true;
         } else if (keyEvent->key() == Qt::Key_End) {
             QTextCursor cursor = textCursor();
             // Meta is Control on macOS
             cursor.movePosition(
-                keyEvent->modifiers().testFlag(Qt::MetaModifier) ?
-                    QTextCursor::End : QTextCursor::EndOfLine,
-                keyEvent->modifiers().testFlag(Qt::ShiftModifier) ?
-                    QTextCursor::KeepAnchor : QTextCursor::MoveAnchor);
+                keyEvent->modifiers().testFlag(Qt::MetaModifier)
+                    ? QTextCursor::End
+                    : QTextCursor::EndOfLine,
+                keyEvent->modifiers().testFlag(Qt::ShiftModifier)
+                    ? QTextCursor::KeepAnchor
+                    : QTextCursor::MoveAnchor);
             this->setTextCursor(cursor);
             return true;
 #endif
@@ -465,7 +465,7 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
     } else if (event->type() == QEvent::MouseButtonDblClick) {
         _mouseButtonDown = true;
     } else if (event->type() == QEvent::Wheel) {
-        auto *wheel = dynamic_cast<QWheelEvent*>(event);
+        auto *wheel = dynamic_cast<QWheelEvent *>(event);
 
         // emit zoom signals
         if (wheel->modifiers() == Qt::ControlModifier) {
@@ -649,8 +649,7 @@ void QMarkdownTextEdit::moveTextUpDown(bool up) {
     setTextCursor(move);
 }
 
-void QMarkdownTextEdit::setLineNumberEnabled(bool enabled)
-{
+void QMarkdownTextEdit::setLineNumberEnabled(bool enabled) {
     _lineNumArea->setLineNumAreaEnabled(enabled);
     updateLineNumberAreaWidth(0);
 }
@@ -715,10 +714,12 @@ bool QMarkdownTextEdit::handleBracketClosing(const QChar openingCharacter,
 
     // get the current text from the block (inserted character not included)
     // Remove whitespace at start of string (e.g. in multilevel-lists).
-    const QString text = cursor.block().text().remove(QRegularExpression("^\\s+"));
+    static QRegularExpression regex1("^\\s+");
+    const QString text = cursor.block().text().remove(regex1);
 
     const int pib = cursor.positionInBlock();
-    bool isPreviousAsterisk = pib > 0 && pib < text.length() && text.at(pib - 1) == '*';
+    bool isPreviousAsterisk =
+        pib > 0 && pib < text.length() && text.at(pib - 1) == '*';
     bool isNextAsterisk = pib < text.length() && text.at(pib) == '*';
     bool isMaybeBold = isPreviousAsterisk && isNextAsterisk;
     if (pib < text.length() && !isMaybeBold && !text.at(pib).isSpace()) {
@@ -759,12 +760,15 @@ bool QMarkdownTextEdit::handleBracketClosing(const QChar openingCharacter,
         }
     }
 
-    // Auto completion for ``` pair
+    // Auto-completion for ``` pair
     if (openingCharacter == QLatin1Char('`')) {
 #if QT_VERSION < QT_VERSION_CHECK(5, 12, 0)
         if (QRegExp(QStringLiteral("[^`]*``")).exactMatch(text)) {
 #else
-        if (QRegularExpression(QRegularExpression::anchoredPattern(QStringLiteral("[^`]*``"))).match(text).hasMatch()) {
+        if (QRegularExpression(
+                QRegularExpression::anchoredPattern(QStringLiteral("[^`]*``")))
+                .match(text)
+                .hasMatch()) {
 #endif
             cursor.insertText(QStringLiteral("``"));
             cursorSubtract = 3;
@@ -955,7 +959,7 @@ bool QMarkdownTextEdit::handleBackspaceEntered() {
         return handleCharRemoval(MarkdownHighlighter::RangeType::CodeSpan,
                                  block, positionInBlock - 1);
 
-    //handle removal of ", ', and brackets
+    // handle removal of ", ', and brackets
 
     // is it opener?
     int pos = _openingCharacters.indexOf(charInFront);
@@ -1020,18 +1024,14 @@ bool QMarkdownTextEdit::handleBackspaceEntered() {
 }
 
 bool QMarkdownTextEdit::handleCharRemoval(MarkdownHighlighter::RangeType type,
-                                          int block, int position)
-{
-    if (!_highlighter)
-        return false;
+                                          int block, int position) {
+    if (!_highlighter) return false;
 
     auto range = _highlighter->findPositionInRanges(type, block, position);
-    if (range == QPair<int, int>{-1, -1})
-        return false;
+    if (range == QPair<int, int>{-1, -1}) return false;
 
     int charToRemovePos = range.first;
-    if (position == range.first)
-        charToRemovePos = range.second;
+    if (position == range.first) charToRemovePos = range.second;
 
     QTextCursor cursor = textCursor();
     auto gpos = cursor.position();
@@ -1049,19 +1049,18 @@ bool QMarkdownTextEdit::handleCharRemoval(MarkdownHighlighter::RangeType type,
     return false;
 }
 
-void QMarkdownTextEdit::updateLineNumAreaGeometry()
-{
+void QMarkdownTextEdit::updateLineNumAreaGeometry() {
     const auto contentsRect = this->contentsRect();
     const QRect newGeometry = {contentsRect.left(), contentsRect.top(),
-                               _lineNumArea->sizeHint().width(), contentsRect.height()};
+                               _lineNumArea->sizeHint().width(),
+                               contentsRect.height()};
     auto oldGeometry = _lineNumArea->geometry();
     if (newGeometry != oldGeometry) {
         _lineNumArea->setGeometry(newGeometry);
     }
 }
 
-void QMarkdownTextEdit::resizeEvent(QResizeEvent *event)
-{
+void QMarkdownTextEdit::resizeEvent(QResizeEvent *event) {
     QPlainTextEdit::resizeEvent(event);
     updateLineNumAreaGeometry();
 }
@@ -1094,7 +1093,6 @@ bool QMarkdownTextEdit::increaseSelectedTextIndention(
         if (reverse) {
             // un-indent text
 
-            //          QSettings settings;
             const int indentSize = indentCharacters == QStringLiteral("\t")
                                        ? 4
                                        : indentCharacters.length();
@@ -1123,7 +1121,8 @@ bool QMarkdownTextEdit::increaseSelectedTextIndention(
                 .prepend(indentCharacters);
 
             // remove trailing \t
-            newText.remove(QRegularExpression(QStringLiteral("\\t$")));
+            static QRegularExpression regex1(QStringLiteral("\\t$"));
+            newText.remove(regex1);
         }
 
         // insert the new text
@@ -1151,8 +1150,8 @@ bool QMarkdownTextEdit::increaseSelectedTextIndention(
             }
 
             // check for \t or space in front of cursor
-            QRegularExpression re(QStringLiteral("[\\t ]"));
-            QRegularExpressionMatch match = re.match(cursor.selectedText());
+            static QRegularExpression regex1(QStringLiteral("[\\t ]"));
+            QRegularExpressionMatch match = regex1.match(cursor.selectedText());
 
             if (!match.hasMatch()) {
                 // (select to) check for \t or space after the cursor
@@ -1163,7 +1162,7 @@ bool QMarkdownTextEdit::increaseSelectedTextIndention(
                 }
             }
 
-            match = re.match(cursor.selectedText());
+            match = regex1.match(cursor.selectedText());
 
             if (match.hasMatch()) {
                 cursor.removeSelectedText();
@@ -1231,8 +1230,8 @@ bool QMarkdownTextEdit::openLinkAtCursorPosition() {
  * @return
  */
 bool QMarkdownTextEdit::isValidUrl(const QString &urlString) {
-    const QRegularExpressionMatch match =
-        QRegularExpression(R"(^\w+:\/\/.+)").match(urlString);
+    static QRegularExpression regex(R"(^\w+:\/\/.+)");
+    const QRegularExpressionMatch match = regex.match(urlString);
     return match.hasMatch();
 }
 
@@ -1284,13 +1283,12 @@ void QMarkdownTextEdit::setIgnoredClickUrlSchemata(
 QMap<QString, QString> QMarkdownTextEdit::parseMarkdownUrlsFromText(
     const QString &text) {
     QMap<QString, QString> urlMap;
-    QRegularExpression regex;
     QRegularExpressionMatchIterator iterator;
 
     // match urls like this: <http://mylink>
     //    re = QRegularExpression("(<(.+?:\\/\\/.+?)>)");
-    regex = QRegularExpression(QStringLiteral("(<(.+?)>)"));
-    iterator = regex.globalMatch(text);
+    static QRegularExpression regex1(QStringLiteral("(<(.+?)>)"));
+    iterator = regex1.globalMatch(text);
     while (iterator.hasNext()) {
         QRegularExpressionMatch match = iterator.next();
         QString linkText = match.captured(1);
@@ -1300,8 +1298,8 @@ QMap<QString, QString> QMarkdownTextEdit::parseMarkdownUrlsFromText(
 
     // match urls like this: [this url](http://mylink)
     //    QRegularExpression re("(\\[.*?\\]\\((.+?:\\/\\/.+?)\\))");
-    regex = QRegularExpression(R"((\[.*?\]\((.+?)\)))");
-    iterator = regex.globalMatch(text);
+    static QRegularExpression regex2(R"((\[.*?\]\((.+?)\)))");
+    iterator = regex2.globalMatch(text);
     while (iterator.hasNext()) {
         QRegularExpressionMatch match = iterator.next();
         QString linkText = match.captured(1);
@@ -1310,8 +1308,8 @@ QMap<QString, QString> QMarkdownTextEdit::parseMarkdownUrlsFromText(
     }
 
     // match urls like this: http://mylink
-    regex = QRegularExpression(R"(\b\w+?:\/\/[^\s]+[^\s>\)])");
-    iterator = regex.globalMatch(text);
+    static QRegularExpression regex3(R"(\b\w+?:\/\/[^\s]+[^\s>\)])");
+    iterator = regex3.globalMatch(text);
     while (iterator.hasNext()) {
         QRegularExpressionMatch match = iterator.next();
         QString url = match.captured(0);
@@ -1319,8 +1317,8 @@ QMap<QString, QString> QMarkdownTextEdit::parseMarkdownUrlsFromText(
     }
 
     // match urls like this: www.github.com
-    regex = QRegularExpression(R"(\bwww\.[^\s]+\.[^\s]+\b)");
-    iterator = regex.globalMatch(text);
+    static QRegularExpression regex4(R"(\bwww\.[^\s]+\.[^\s]+\b)");
+    iterator = regex4.globalMatch(text);
     while (iterator.hasNext()) {
         QRegularExpressionMatch match = iterator.next();
         QString url = match.captured(0);
@@ -1329,8 +1327,8 @@ QMap<QString, QString> QMarkdownTextEdit::parseMarkdownUrlsFromText(
 
     // match reference urls like this: [this url][1] with this later:
     // [1]: http://domain
-    regex = QRegularExpression(R"((\[.*?\]\[(.+?)\]))");
-    iterator = regex.globalMatch(text);
+    static QRegularExpression regex5(R"((\[.*?\]\[(.+?)\]))");
+    iterator = regex5.globalMatch(text);
     while (iterator.hasNext()) {
         QRegularExpressionMatch match = iterator.next();
         QString linkText = match.captured(1);
@@ -1435,8 +1433,7 @@ void QMarkdownTextEdit::setText(const QString &text) { setPlainText(text); }
 void QMarkdownTextEdit::setPlainText(const QString &text) {
     // clear the dirty blocks vector to increase performance and prevent
     // a possible crash in QSyntaxHighlighter::rehighlightBlock
-    if (_highlighter)
-        _highlighter->clearDirtyBlocks();
+    if (_highlighter) _highlighter->clearDirtyBlocks();
 
     QPlainTextEdit::setPlainText(text);
     adjustRightMargin();
@@ -1481,18 +1478,27 @@ bool QMarkdownTextEdit::handleReturnEntered() {
         return true;
     }
 
+    // This will be the main cursor to add or remove text
     QTextCursor cursor = this->textCursor();
-    const int position = cursor.position();
 
+    // We need a 2nd cursor to get the text of the current block without moving
+    // the main cursor that is used to remove the selected text
+    QTextCursor cursor2 = this->textCursor();
+    cursor2.select(QTextCursor::BlockUnderCursor);
+    const QString currentLineText = cursor2.selectedText().trimmed();
+
+    const int position = cursor.position();
+    const bool cursorAtBlockStart = cursor.atBlockStart();
     cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
-    const QString currentLineText = cursor.selectedText();
+    const QString currentLinePartialText = cursor.selectedText();
 
     // if return is pressed and there is just an unordered list symbol then we
     // want to remove the list symbol Valid listCharacters: '+ ', '-' , '* ', '+
     // [ ] ', '+ [x] ', '- [ ] ', '- [-] ', '- [x] ', '* [ ] ', '* [x] '.
-    QRegularExpression regex(R"(^(\s*)([+|\-|\*] \[(x|-| |)\]|[+\-\*])(\s+)$)");
+    static QRegularExpression regex1(
+        R"(^(\s*)([+|\-|\*] \[(x|-| |)\]|[+\-\*])(\s+)$)");
     QRegularExpressionMatchIterator iterator =
-        regex.globalMatch(currentLineText);
+        regex1.globalMatch(currentLinePartialText);
     if (iterator.hasNext()) {
         cursor.removeSelectedText();
         return true;
@@ -1500,8 +1506,8 @@ bool QMarkdownTextEdit::handleReturnEntered() {
 
     // if return is pressed and there is just an ordered list symbol then we
     // want to remove the list symbol
-    regex = QRegularExpression(R"(^(\s*)(\d+[\.|\)])(\s+)$)");
-    iterator = regex.globalMatch(currentLineText);
+    static QRegularExpression regex2(R"(^(\s*)(\d+[\.|\)])(\s+)$)");
+    iterator = regex2.globalMatch(currentLinePartialText);
     if (iterator.hasNext()) {
         qDebug() << cursor.selectedText();
         cursor.removeSelectedText();
@@ -1512,13 +1518,11 @@ bool QMarkdownTextEdit::handleReturnEntered() {
     // We are in a list when we have '* ', '- ' or '+ ', possibly with preceding
     // whitespace. If e.g. user has entered '**text**' and pressed enter - we
     // don't want to do more list-stuff.
-    QString currentLine = currentLineText.trimmed();
+    QString currentLine = currentLinePartialText.trimmed();
     QChar char0;
     QChar char1;
-    if (currentLine.length() >= 1)
-        char0 = currentLine.at(0);
-    if (currentLine.length() >= 2)
-        char1 = currentLine.at(1);
+    if (currentLine.length() >= 1) char0 = currentLine.at(0);
+    if (currentLine.length() >= 2) char1 = currentLine.at(1);
     const bool inList =
         ((char0 == QLatin1Char('*') || char0 == QLatin1Char('-') ||
           char0 == QLatin1Char('+')) &&
@@ -1529,21 +1533,21 @@ bool QMarkdownTextEdit::handleReturnEntered() {
         // whitespaces) add the whitespaces at the next line too
         // Valid listCharacters: '+ ', '-' , '* ', '+ [ ] ', '+ [x] ', '- [ ] ',
         // '- [x] ', '- [-] ', '* [ ] ', '* [x] '.
-        regex =
-            QRegularExpression(R"(^(\s*)([+|\-|\*] \[(x|-| |)\]|[+\-\*])(\s+))");
-        iterator = regex.globalMatch(currentLineText);
+        static QRegularExpression regex3(
+            R"(^(\s*)([+|\-|\*] \[(x|-| |)\]|[+\-\*])(\s+))");
+        iterator = regex3.globalMatch(currentLinePartialText);
         if (iterator.hasNext()) {
             const QRegularExpressionMatch match = iterator.next();
             const QString whitespaces = match.captured(1);
             QString listCharacter = match.captured(2);
             const QString whitespaceCharacter = match.captured(4);
 
+            static QRegularExpression regex4(R"(^([+|\-|\*]) \[(x| |\-|)\])");
             // start new checkbox list item with an unchecked checkbox
-            iterator = QRegularExpression(R"(^([+|\-|\*]) \[(x| |)\])")
-                           .globalMatch(listCharacter);
+            iterator = regex4.globalMatch(listCharacter);
             if (iterator.hasNext()) {
-                const QRegularExpressionMatch match = iterator.next();
-                const QString realListCharacter = match.captured(1);
+                const QRegularExpressionMatch match1 = iterator.next();
+                const QString realListCharacter = match1.captured(1);
                 listCharacter = realListCharacter + QStringLiteral(" [ ]");
             }
 
@@ -1558,8 +1562,8 @@ bool QMarkdownTextEdit::handleReturnEntered() {
     }
 
     // check for ordered lists and increment the list number in the next line
-    regex = QRegularExpression(R"(^(\s*)(\d+)([\.|\)])(\s+))");
-    iterator = regex.globalMatch(currentLineText);
+    static QRegularExpression regex5(R"(^(\s*)(\d+)([\.|\)])(\s+))");
+    iterator = regex5.globalMatch(currentLinePartialText);
     if (iterator.hasNext()) {
         const QRegularExpressionMatch match = iterator.next();
         const QString whitespaces = match.captured(1);
@@ -1577,8 +1581,8 @@ bool QMarkdownTextEdit::handleReturnEntered() {
     }
 
     // intent next line with same whitespaces as in current line
-    regex = QRegularExpression(R"(^(\s+))");
-    iterator = regex.globalMatch(currentLineText);
+    static QRegularExpression regex6(R"(^(\s+))");
+    iterator = regex6.globalMatch(currentLinePartialText);
     if (iterator.hasNext()) {
         const QRegularExpressionMatch match = iterator.next();
         const QString whitespaces = match.captured(1);
@@ -1591,6 +1595,39 @@ bool QMarkdownTextEdit::handleReturnEntered() {
         return true;
     }
 
+    // Add new list item above current line if we are at the start the line of a
+    // list item
+    if (cursorAtBlockStart) {
+        static QRegularExpression regex7(
+            R"(^([+|\-|\*] \[(x|-| |)\]|[+\-\*])(\s+))");
+        iterator = regex7.globalMatch(currentLineText);
+        if (iterator.hasNext()) {
+            const QRegularExpressionMatch match = iterator.next();
+            QString listCharacter = match.captured(1);
+            const QString whitespaceCharacter = match.captured(3);
+
+            static QRegularExpression regex8(R"(^([+|\-|\*]) \[(x| |\-|)\])");
+            // start new checkbox list item with an unchecked checkbox
+            iterator = regex8.globalMatch(listCharacter);
+            if (iterator.hasNext()) {
+                const QRegularExpressionMatch match1 = iterator.next();
+                const QString realListCharacter = match1.captured(1);
+                listCharacter = realListCharacter + QStringLiteral(" [ ]");
+            }
+
+            cursor.setPosition(position);
+            // Enter new list item above current line
+            cursor.insertText(listCharacter + whitespaceCharacter + "\n");
+            // Move the cursor at the end of the new list item
+            cursor.movePosition(QTextCursor::Left);
+            setTextCursor(cursor);
+
+            // scroll to the cursor if we are at the bottom of the document
+            ensureCursorVisible();
+            return true;
+        }
+    }
+
     return false;
 }
 
@@ -1613,8 +1650,9 @@ bool QMarkdownTextEdit::handleTabEntered(bool reverse,
         // check if we want to indent or un-indent an ordered list
         // Valid listCharacters: '+ ', '-' , '* ', '+ [ ] ', '+ [x] ', '- [ ] ',
         // '- [x] ', '- [-] ', '* [ ] ', '* [x] '.
-        QRegularExpression re(R"(^(\s*)([+|\-|\*] \[(x|-| )\]|[+\-\*])(\s+)$)");
-        QRegularExpressionMatchIterator i = re.globalMatch(currentLineText);
+        static QRegularExpression regex1(
+            R"(^(\s*)([+|\-|\*] \[(x|-| )\]|[+\-\*])(\s+)$)");
+        QRegularExpressionMatchIterator i = regex1.globalMatch(currentLineText);
 
         if (i.hasNext()) {
             QRegularExpressionMatch match = i.next();
@@ -1640,8 +1678,8 @@ bool QMarkdownTextEdit::handleTabEntered(bool reverse,
         }
 
         // check if we want to indent or un-indent an ordered list
-        re = QRegularExpression(R"(^(\s*)(\d+)([\.|\)])(\s+)$)");
-        i = re.globalMatch(currentLineText);
+        static QRegularExpression regex2(R"(^(\s*)(\d+)([\.|\)])(\s+)$)");
+        i = regex2.globalMatch(currentLineText);
 
         if (i.hasNext()) {
             const QRegularExpressionMatch match = i.next();
@@ -1674,12 +1712,12 @@ void QMarkdownTextEdit::setAutoTextOptions(AutoTextOptions options) {
     _autoTextOptions = options;
 }
 
-void QMarkdownTextEdit::updateLineNumberArea(const QRect rect, int dy)
-{
+void QMarkdownTextEdit::updateLineNumberArea(const QRect rect, int dy) {
     if (dy)
         _lineNumArea->scroll(0, dy);
     else
-        _lineNumArea->update(0, rect.y(), _lineNumArea->sizeHint().width(), rect.height());
+        _lineNumArea->update(0, rect.y(), _lineNumArea->sizeHint().width(),
+                             rect.height());
 
     updateLineNumAreaGeometry();
 
@@ -1688,14 +1726,15 @@ void QMarkdownTextEdit::updateLineNumberArea(const QRect rect, int dy)
     }
 }
 
-void QMarkdownTextEdit::updateLineNumberAreaWidth(int)
-{
+void QMarkdownTextEdit::updateLineNumberAreaWidth(int) {
     QSignalBlocker blocker(this);
     const auto oldMargins = viewportMargins();
-    const int width = _lineNumArea->isLineNumAreaEnabled() ?
-           _lineNumArea->sizeHint().width() + _lineNumberLeftMarginOffset :
-           oldMargins.left();
-    const auto newMargins = QMargins{width, oldMargins.top(), oldMargins.right(), oldMargins.bottom()};
+    const int width =
+        _lineNumArea->isLineNumAreaEnabled()
+            ? _lineNumArea->sizeHint().width() + _lineNumberLeftMarginOffset
+            : oldMargins.left();
+    const auto newMargins = QMargins{width, oldMargins.top(),
+                                     oldMargins.right(), oldMargins.bottom()};
 
     if (newMargins != oldMargins) {
         setViewportMargins(newMargins);
@@ -1841,12 +1880,14 @@ void QMarkdownTextEdit::paintEvent(QPaintEvent *e) {
         // Current line highlight
         QTextCursor cursor = textCursor();
         if (highlightCurrentLine() && cursor.block() == block) {
-            QTextLine line = block.layout()->lineForTextPosition(cursor.positionInBlock());
+            QTextLine line =
+                block.layout()->lineForTextPosition(cursor.positionInBlock());
             QRectF lineRect = line.rect();
             lineRect.moveTop(lineRect.top() + r.top());
             lineRect.setLeft(0.);
             lineRect.setRight(viewportRect.width());
-            painter.fillRect(lineRect.toAlignedRect(), currentLineHighlightColor());
+            painter.fillRect(lineRect.toAlignedRect(),
+                             currentLineHighlightColor());
         }
 
         block = block.next();
